"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.makeUnrecognizedError = exports.makeKnownError = exports.makeConnectionError = exports.isTimeoutError = exports.isServiceBusyError = exports.isConnectionTerminatedError = void 0;

var _eventstoreBase = require("@resolve-js/eventstore-base");

const checkFormalError = (error, value) => error.name === value || error.code === value;

const checkFuzzyError = (error, value) => value.test(error.message) || value.test(error.stack); // https://www.postgresql.org/docs/10/errcodes-appendix.html


const isConnectionTerminatedError = error => {
  return error != null && (checkFuzzyError(error, /Connection terminated/i) || checkFuzzyError(error, /terminating connection due to serverless scale event timeout/i) || checkFuzzyError(error, /terminating connection due to administrator command/i) || checkFuzzyError(error, /Client has encountered a connection error and is not queryable/i) || checkFormalError(error, 'ECONNRESET') || checkFormalError(error, '08000') || checkFormalError(error, '08003') || checkFormalError(error, '08006'));
};

exports.isConnectionTerminatedError = isConnectionTerminatedError;

const isTimeoutError = error => {
  return error != null && (checkFuzzyError(error, /canceling statement due to statement timeout/i) || checkFuzzyError(error, /Query read timeout/i) || checkFuzzyError(error, /timeout expired/i) || checkFormalError(error, 'ETIMEDOUT'));
};

exports.isTimeoutError = isTimeoutError;

const isServiceBusyError = error => {
  return error != null && (checkFuzzyError(error, /too many clients already/i) || checkFuzzyError(error, /Connection rate is too high, please reduce connection rate/i) || checkFormalError(error, '55P03') || checkFuzzyError(error, /could not obtain lock on relation/) || checkFuzzyError(error, /getaddrinfo/) || checkFuzzyError(error, /SQLState: 08001/) || checkFuzzyError(error, /remaining connection slots are reserved/i));
};

exports.isServiceBusyError = isServiceBusyError;

function extendErrorStack(mainError, origError) {
  if (origError.stack) {
    if (mainError.stack) {
      mainError.stack += '\n';
      mainError.stack += origError.stack;
    } else {
      mainError.stack = origError.stack;
    }
  }
}

const makeKnownError = error => {
  if (isServiceBusyError(error) || isConnectionTerminatedError(error)) {
    const busyError = new _eventstoreBase.ServiceBusyError(error.message);
    extendErrorStack(busyError, error);
    return busyError;
  } else if (isTimeoutError(error)) {
    const timeoutError = new _eventstoreBase.RequestTimeoutError(error.message);
    extendErrorStack(timeoutError, error);
    return timeoutError;
  } else {
    return null;
  }
};

exports.makeKnownError = makeKnownError;

const makeConnectionError = error => {
  const knownError = makeKnownError(error);

  if (knownError !== null) {
    return knownError;
  } else {
    const connectionError = new _eventstoreBase.ConnectionError(error.message);
    extendErrorStack(connectionError, error);
    return connectionError;
  }
};

exports.makeConnectionError = makeConnectionError;

const makeUnrecognizedError = error => {
  if (error) {
    const unrecognizedError = new _eventstoreBase.UnrecognizedError(error.message);
    unrecognizedError.code = error.code;
    extendErrorStack(unrecognizedError, error);
    return unrecognizedError;
  } else {
    return new _eventstoreBase.UnrecognizedError('unrecognized error');
  }
};

exports.makeUnrecognizedError = makeUnrecognizedError;
//# sourceMappingURL=errors.js.map