"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _eventstoreBase = require("@resolve-js/eventstore-base");

var _errors = require("./errors");

var _resourceErrors = require("./resource-errors");

var _constants = require("./constants");

var _makePostgresClient = _interopRequireDefault(require("./make-postgres-client"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const DISPOSED_WHILE_RUNNING_MSG = 'Adapter disposed while operation is running';

const executeStatement = async (pool, sql, distinctConnection) => {
  var _pool$maxReconnection, _pool$delayBeforeReco;

  let reconnectionTimes = 0;
  let useDistinctConnection = distinctConnection;
  let distinctConnectionMade = false;

  const incrementReconnectionTimes = () => {
    reconnectionTimes++;

    if (pool.monitoring) {
      pool.monitoring.custom({
        metricName: 'EventstorePostgresqlReconnections',
        unit: 'Count'
      });
    }
  };

  const maxReconnectionTimes = (_pool$maxReconnection = pool.maxReconnectionTimes) !== null && _pool$maxReconnection !== void 0 ? _pool$maxReconnection : _constants.MAX_RECONNECTIONS;
  const delayBeforeReconnection = (_pool$delayBeforeReco = pool.delayBeforeReconnection) !== null && _pool$delayBeforeReco !== void 0 ? _pool$delayBeforeReco : _constants.SERVICE_WAIT_TIME;

  while (true) {
    if (pool.disposed) {
      throw new _eventstoreBase.AlreadyDisposedError(DISPOSED_WHILE_RUNNING_MSG);
    }

    let connection;

    try {
      if (useDistinctConnection) {
        connection = (0, _makePostgresClient.default)(pool);
        await connection.connect();
        pool.extraConnections.add(connection);
        distinctConnectionMade = true;
      } else {
        connection = await pool.getConnectPromise();
      }
    } catch (error) {
      if (!useDistinctConnection) {
        pool.getConnectPromise = pool.createGetConnectPromise();
      }

      if ((0, _errors.isConnectionTerminatedError)(error) || (0, _errors.isServiceBusyError)(error)) {
        if (reconnectionTimes >= maxReconnectionTimes) {
          throw new _eventstoreBase.ServiceBusyError(error.message);
        }

        incrementReconnectionTimes();
        await new Promise(resolve => setTimeout(resolve, delayBeforeReconnection));
        continue;
      } else {
        throw (0, _errors.makeConnectionError)(error);
      }
    }

    let shouldWaitForServiceFree = false;

    try {
      if (pool.disposed) {
        throw new _eventstoreBase.AlreadyDisposedError(DISPOSED_WHILE_RUNNING_MSG);
      }

      const result = await connection.query(sql);

      if (result != null && Array.isArray(result.rows)) {
        return result.rows;
      }

      return [];
    } catch (error) {
      if ((0, _resourceErrors.isAlreadyExistsError)(error) || (0, _resourceErrors.isNotExistError)(error) || _eventstoreBase.AlreadyDisposedError.is(error)) {
        throw error;
      } else if ((0, _errors.isServiceBusyError)(error)) {
        throw new _eventstoreBase.ServiceBusyError(error.message);
      } else if ((0, _errors.isTimeoutError)(error)) {
        throw new _eventstoreBase.RequestTimeoutError(error.message);
      } else if ((0, _errors.isConnectionTerminatedError)(error)) {
        if (!useDistinctConnection) {
          pool.getConnectPromise = pool.createGetConnectPromise();
        }

        if (reconnectionTimes >= maxReconnectionTimes) {
          throw new _eventstoreBase.ServiceBusyError(error.message + `${useDistinctConnection}`);
        }

        useDistinctConnection = true;
        shouldWaitForServiceFree = true;
        incrementReconnectionTimes();
      } else if (error != null && error.message === 'Client was closed and is not queryable') {
        if (pool.disposed) {
          throw new _eventstoreBase.AlreadyDisposedError(DISPOSED_WHILE_RUNNING_MSG);
        }

        if (reconnectionTimes >= maxReconnectionTimes) {
          throw new _eventstoreBase.ServiceBusyError(error.message);
        }

        useDistinctConnection = true;
        shouldWaitForServiceFree = true;
        incrementReconnectionTimes();
      } else {
        throw (0, _errors.makeUnrecognizedError)(error);
      }
    } finally {
      if (distinctConnectionMade) {
        pool.extraConnections.delete(connection);
        await connection.end();
      }

      if (shouldWaitForServiceFree) {
        await new Promise(resolve => setTimeout(resolve, delayBeforeReconnection));
      }
    }
  }
};

var _default = executeStatement;
exports.default = _default;
//# sourceMappingURL=execute-statement.js.map